local MAJOR = "LibScriptableDisplayPluginTalents-1.0"
local MINOR = 14
local PluginTalents = LibStub:NewLibrary(MAJOR, MINOR)
if not PluginTalents then return end
local GroupTalents = LibStub("LibGroupTalents-1.0", true)
assert(GroupTalents, MAJOR .. " requires LibGroupTalents-1.0")
local LibTimer = LibStub("LibScriptableDisplayTimer-1.0", true)
assert(LibTimer, MAJOR .. " requires LibScriptableDisplayTimer-1.0")
local L = LibStub("LibScriptableDisplayLocale-1.0", true)
assert(L, MAJOR .. " requires LibScriptableDisplayLocale-1.0")
L = L.L

local _G = _G
local GameTooltip = _G.GameTooltip
local UnitIsUnit = _G.UnitIsUnit
local GetNumTalentTabs = _G.GetNumTalentTabs
local GetTalentTabInfo = _G.GetTalentTabInfo
local UnitExists = _G.UnitExists
local UnitIsPlayer = _G.UnitIsPlayer
local UnitName = _G.UnitName
local EXPIRE_TIME = 5000
local spec = setmetatable({}, {__mode='v'})
local frame = CreateFrame("Frame")
local count = 0
local query = {}

if not PluginTalents.__index then
	PluginTalents.__index = PluginTalents
end

-- @name PluginTalents:New
-- @usage :New(environment)
-- @parma environment This will be the environment when loadstring is callled.
-- @return A new PluginTalents object
function PluginTalents:New(environment)

	environment.SpecText = self.SpecText
	environment.GetSpec = self.GetSpec
	environment.ClearSpec = self.ClearSpec
	environment.SendQuery = self.SendQuery
	
	GroupTalents.RegisterCallback(self, "LibGroupTalents_Update", "OnUpdate")
	GroupTalents.RegisterCallback(self, "LibGroupTalents_Add", "OnAdd")
	GroupTalents.RegisterCallback(self, "LibGroupTalents_UpdateComplete", "OnUpdateComplete")
	GroupTalents.RegisterCallback(self, "LibGroupTalents_RoleChange", "OnRoleChange")
end

local talentTrees = {
	[L["Druid"]] = {L["Balance"], L["Feral Combat"], L["Restoration"]},
	[L["Hunter"]] = {L["Beast Mastery"], L["Marksmanship"], L["Survival"]},
	[L["Mage"]] = {L["Arcane"], L["Fire"], L["Frost"]},
	[L["Paladin"]] = {L["Holy"], L["Protection"], L["Retribution"]},
	[L["Priest"]] = {L["Discipline"], L["Holy"], L["Shadow"]},
	[L["Rogue"]] = {L["Assassination"], L["Combat"], L["Subtlety"]},
	[L["Shaman"]] = {L["Elemental"], L["Enhancement"], L["Restoration"]},
	[L["Warlock"]] = {L["Affliction"], L["Demonology"], L["Destruction"]},
	[L["Warrior"]] = {L["Arms"], L["Fury"], L["Protection"]},
}

local indexOf = function(t, val)
	for i=1, #t do
		if t[i] == val then
			return i
		end
	end
end

local indicesOf = function(t, val)
	local a = {}
	for i=1, #t do
		if t[i] == val then
			tinsert(a, i)
		end
	end
	return unpack(a)
end

function PluginTalents:OnAdd(guid, unit, name, realm)
	StarTip:Print("OnADd")
	self:OnUpdate(guid, unit)
end

local function sortfunc(a,b) return a>b end

function PluginTalents:OnUpdate(guid, unitid)
	if not unitid then return end
	StarTip:Print("OnUpdate")
	local talent1, talent2, talent3 = GroupTalents:GetUnitTalents(unitid, true)
	if not talent1 then return end
	
	local class = UnitClass(unitid)
	local spec1, spec2, spec3 = GroupTalents:GetTreeNames(class)
	local guid = UnitGUID(unitid)
	if not spec[guid] then
		spec[guid] = {[4]=NONE}
		local highPoints = {}
		local specNames = {}
		for i, treename in ipairs({spec1, spec2, spec3}) do
			local treename = v
			local pointsspent = {talent1, talent2, talent3}
			highPoints[i] = pointsspent
			spec[guid][i] = pointsspent
			specNames[i] = treename
		end
		if highPoints[1] == nil or highPoints[2] == nil or highPoints[3] == nil then spec[guid] = nil return end
		table.sort(highPoints, sortfunc)
		local first, second = select(1, indicesOf(spec[guid], highPoints[1])), select(2, indicesOf(spec[guid], highPoints[1]))
		if highPoints[1] > 0 and highPoints[2] > 0 and highPoints[1] - highPoints[2] <= 5 and highPoints[1] ~= highPoints[2] then
			spec[guid][4] = specNames[indexOf(spec[guid], highPoints[1])] .. "/" .. specNames[indexOf(spec[guid], highPoints[2])]
		elseif highPoints[1] > 0 and first and second then
			spec[guid][4] = specNames[first] .. "/" .. specNames[second]
		elseif highPoints[1] > 0 then
			spec[guid][4] = specNames[indexOf(spec[guid], highPoints[1])]
		end
	end
	query[guid] = false
	if expireTimer then expireTimer:Stop() end
	if expireExpireTimer then expireExpireTimer:Stop() end
	expired = false
end

function PluginTalents:OnUpdateComplete(...)
	for i = 1, select("#", ...) do
		local unit = select(i, ...)
		PluginTalents.OnUpdate(UnitGUID(unit), unit) 
	end
end

function PluginTalents:OnRoleChange(guid, unit)
	self:OnUpdate(guid, unit)
end

function PluginTalents.SendQuery(unit)
	if UnitIsPlayer(unit) then
		GroupTalents:RefreshTalentsByUnit(unit)
	end
end

function PluginTalents.SpecText(unit)
	if type(unit) ~= "string" then return end
	if not UnitExists(unit) then return L["Unknown"] end
	if not UnitIsPlayer(unit) then return end
	local guid = UnitGUID(unit)
	
	if expired then return L["Expired"] end
		
	local periods = ""
	for i = 0, count % 3 do
		periods = periods .. "."
	end
	count = count + 1
	if not spec[guid] then return L["Loading"] .. periods end
	return ('%s (%d/%d/%d)'):format(spec[guid][4], spec[guid][1], spec[guid][2], spec[guid][3])
end

function PluginTalents.GetSpec(unit)
	if type(unit) ~= "string" or not UnitExists(unit) then return end
	local guid = UnitGUID(unit)
		
	if not spec[guid] then return nil end
	return unpack(spec[guid])
end

function PluginTalents.ClearSpec(unit)
	if type(unit) ~= "string" or not UnitExists(unit) then return end
	local guid = UnitGUID(unit)
	spec[guid] = nil
end


